@@ -13,7 +13,8 @@ import pathlib
 import re
 import subprocess
 import sys
-from glob import glob
+from datetime import date
+from pathlib import Path
 
 import requests
 from coverage.config import CoverageConfig
@@ -30,17 +31,18 @@ BUILD_FILES = tuple(
     os.path.join(tools.ROOT, f)
     for f in ["tooling", "requirements", ".github", "hypothesis-python/tox.ini"]
 )
+TODAY = date.today().isoformat()
 
 
 def task(if_changed=()):
-    if isinstance(if_changed, str):
+    if not isinstance(if_changed, tuple):
         if_changed = (if_changed,)
 
     def accept(fn):
         def wrapped(*args, **kwargs):
             if if_changed and tools.IS_PULL_REQUEST:
                 if not tools.has_changes(if_changed + BUILD_FILES):
-                    changed = ", ".join(if_changed)
+                    changed = ", ".join(map(str, if_changed))
                     print(f"Skipping task due to no changes in {changed}")
                     return
             fn(*args, **kwargs)
@@ -65,7 +67,11 @@ def codespell(*files):
         "codespell",
         "--check-hidden",
         "--check-filenames",
-        "--ignore-words=./tooling/ignore-list.txt",
+        "--ignore-words=./tooling/codespell-ignore.txt",
+        # passing a custom --dictionary disables the default dictionary by default.
+        # Add it back in with --dictionary=-.
+        "--dictionary=-",
+        "--dictionary=./tooling/codespell-dict.txt",
         "--skip=__pycache__,.mypy_cache,.venv,.git,tlds-alpha-by-domain.txt",
         *files,
     )
@@ -73,12 +79,7 @@ def codespell(*files):
 
 @task()
 def lint():
-    pip_tool(
-        "flake8",
-        *(f for f in tools.all_files() if f.endswith(".py")),
-        "--config",
-        os.path.join(tools.ROOT, ".flake8"),
-    )
+    pip_tool("ruff", "check", ".")
     codespell(*(f for f in tools.all_files() if not f.endswith("by-domain.txt")))
 
 
@@ -103,7 +104,6 @@ def do_release(package):
     tag_name = package.tag_name()
 
     print(f"Creating tag {tag_name}")
-
     tools.create_tag(tag_name)
     tools.push_tag(tag_name)
 
@@ -148,6 +148,33 @@ HEADER = """
 # obtain one at https://mozilla.org/MPL/2.0/.
 """.strip()
 
+# this pattern is copied from shed
+# https://github.com/Zac-HD/shed/blob/6471da71c5b5cc443443ef5ed072799db275e7c0/src/shed/__init__.py#L297
+rst_pattern = re.compile(
+    r"(?P<before>"
+    r"^(?P<indent> *)\.\. "
+    r"(?P<block>jupyter-execute::|"
+    r"invisible-code-block: python|"  # magic rst comment for Sybil doctests
+    r"(code|code-block|sourcecode|ipython):: (python|py|sage|python3|py3|numpy))\n"
+    r"((?P=indent) +:.*\n)*"
+    r"\n*"
+    r")"
+    r"(?P<code>(^((?P=indent) +.*)?\n)+)",
+    flags=re.MULTILINE,
+)
+
+
+def remove_consecutive_newlines_in_rst(path):
+    # replace 2+ empty lines in `.. code-block:: python` blocks with just one empty
+    # line
+    path = Path(path)
+    content = path.read_text()
+    processed_content = rst_pattern.sub(
+        lambda m: m["before"] + re.sub(r"\n{3,}", "\n\n", m["code"]), content
+    )
+    if processed_content != content:
+        path.write_text(processed_content)
+
 
 @task()
 def format():
@@ -213,7 +240,11 @@ def format():
             o.write("\n")
 
     codespell("--write-changes", *files_to_format, *doc_files_to_format)
-    pip_tool("shed", *files_to_format, *doc_files_to_format)
+    pip_tool("ruff", "check", "--fix-only", ".")
+    pip_tool("shed", "--py39-plus", *files_to_format, *doc_files_to_format)
+
+    for f in doc_files_to_format:
+        remove_consecutive_newlines_in_rst(f)
 
 
 VALID_STARTS = (HEADER.split()[0], "#!/usr/bin/env python")
@@ -241,43 +272,70 @@ def check_not_changed():
 
 
 @task()
-def compile_requirements(upgrade=False):
+def compile_requirements(*, upgrade=False):
     if upgrade:
         extra = ["--upgrade", "--rebuild"]
     else:
         extra = []
 
-    for f in glob(os.path.join("requirements", "*.in")):
-        base, _ = os.path.splitext(f)
+    for f in Path("requirements").glob("*.in"):
+        out_file = f.with_suffix(".txt")
         pip_tool(
             "pip-compile",
             "--allow-unsafe",  # future default, not actually unsafe
             "--resolver=backtracking",  # new pip resolver, default in pip-compile 7+
             *extra,
-            f,
-            "hypothesis-python/setup.py",
+            str(f),
+            "hypothesis-python/pyproject.toml",
             "--output-file",
-            base + ".txt",
+            str(out_file),
             cwd=tools.ROOT,
             env={
                 "CUSTOM_COMPILE_COMMAND": "./build.sh upgrade-requirements",
                 **os.environ,
             },
         )
+        # Check that we haven't added anything to output files without adding to inputs
+        out_pkgs = out_file.read_text(encoding="utf-8")
+        for p in f.read_text(encoding="utf-8").splitlines():
+            p = p.lower().replace("_", "-")
+            if re.fullmatch(r"[a-z-]+", p):
+                assert p + "==" in out_pkgs, f"Package `{p}` deleted from {out_file}"
+        out_file.write_text(out_pkgs.replace(f"{tools.ROOT}/", ""))
 
 
 def update_python_versions():
     install.ensure_python(PYTHONS[ci_version])
-    cmd = "~/.cache/hypothesis-build-runtimes/pyenv/bin/pyenv install --list"
-    result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE).stdout.decode()
+    where = os.path.expanduser("~/.cache/hypothesis-build-runtimes/pyenv/")
+    subprocess.run(
+        "git fetch && git reset --hard origin/master",
+        cwd=where,
+        shell=True,
+        capture_output=True,
+    )
+    cmd = "bin/pyenv install --list"
+    result = subprocess.run(
+        cmd, shell=True, stdout=subprocess.PIPE, cwd=where
+    ).stdout.decode()
     # pyenv reports available versions in chronological order, so we keep the newest
     # *unless* our current ends with a digit (is stable) and the candidate does not.
-    stable = re.compile(r".*3\.\d+.\d+$")
+    # (plus some special cases for the `t` suffix for free-threading builds)
+    stable = re.compile(r".*3\.\d+.\d+t?$")
+    min_minor_version = re.search(
+        r'requires-python = ">= ?3.(\d+)"',
+        Path("hypothesis-python/pyproject.toml").read_text(encoding="utf-8"),
+    ).group(1)
     best = {}
     for line in map(str.strip, result.splitlines()):
-        if m := re.match(r"(?:pypy)?3\.(?:[789]|\d\d)", line):
+        if m := re.match(r"(?:pypy)?3\.(?:9|\d\dt?)", line):
             key = m.group()
-            if stable.match(line) or not stable.match(best.get(key, line)):
+            curr = best.get(key, line)
+            if (
+                (stable.match(line) or not stable.match(curr))
+                and not (line.endswith("-dev") and not curr.endswith("-dev"))
+                and int(key.split(".")[-1].rstrip("t")) >= int(min_minor_version)
+                and key.endswith("t") == line.endswith(("t", "t-dev"))
+            ):
                 best[key] = line
 
     if best == PYTHONS:
@@ -285,21 +343,134 @@ def update_python_versions():
 
     # Write the new mapping back to this file
     thisfile = pathlib.Path(__file__)
-    before = thisfile.read_text()
+    before = thisfile.read_text(encoding="utf-8")
     after = re.sub(r"\nPYTHONS = \{[^{}]+\}", f"\nPYTHONS = {best}", before)
-    thisfile.write_text(after)
+    thisfile.write_text(after, encoding="utf-8")
     pip_tool("shed", str(thisfile))
 
     # Automatically sync ci_version with the version in build.sh
-    build_sh = pathlib.Path(tools.ROOT) / "build.sh"
-    sh_before = build_sh.read_text()
+    build_sh = tools.ROOT / "build.sh"
+    sh_before = build_sh.read_text(encoding="utf-8")
     sh_after = re.sub(r"3\.\d\d?\.\d\d?", best[ci_version], sh_before)
     if sh_before != sh_after:
         build_sh.unlink()  # so bash doesn't reload a modified file
-        build_sh.write_text(sh_after)
+        build_sh.write_text(sh_after, encoding="utf-8")
         build_sh.chmod(0o755)
 
 
+DJANGO_VERSIONS = {
+    "4.2": "4.2.20",
+    "5.1": "5.1.8",
+    "5.2": "5.2",
+}
+
+
+def update_django_versions():
+    # https://endoflife.date/django makes it easier to track these
+    releases = requests.get("https://endoflife.date/api/django.json").json()
+    versions = {r["cycle"]: r["latest"] for r in releases[::-1] if TODAY <= r["eol"]}
+
+    if versions == DJANGO_VERSIONS:
+        return
+
+    # Write the new mapping back to this file
+    thisfile = pathlib.Path(__file__)
+    before = thisfile.read_text(encoding="utf-8")
+    after = re.sub(
+        r"DJANGO_VERSIONS = \{[^{}]+\}",
+        "DJANGO_VERSIONS = " + repr(versions).replace("}", ",}"),
+        before,
+    )
+    thisfile.write_text(after, encoding="utf-8")
+    pip_tool("shed", str(thisfile))
+
+    # Update the minimum version in pyproject.toml
+    pyproject_toml = hp.BASE_DIR / "pyproject.toml"
+    content = re.sub(
+        r"django>=\d+\.\d+",
+        f"django>={min(versions, key=float)}",
+        pyproject_toml.read_text(encoding="utf-8"),
+    )
+    pyproject_toml.write_text(content, encoding="utf-8")
+
+    # Automatically sync ci_version with the version in build.sh
+    tox_ini = hp.BASE_DIR / "tox.ini"
+    content = tox_ini.read_text(encoding="utf-8")
+    print(versions)
+    for short, full in versions.items():
+        content = re.sub(
+            rf"django=={short}(\.\d+)?",
+            rf"django=={full}",
+            content,
+        )
+    tox_ini.write_text(content, encoding="utf-8")
+
+
+def update_pyodide_versions():
+
+    def version_tuple(v: str) -> tuple[int, int, int]:
+        return tuple(int(x) for x in v.split("."))  # type: ignore
+
+    vers_re = r"(\d+\.\d+\.\d+)"
+    all_pyodide_build_versions = re.findall(
+        f"pyodide_build-{vers_re}-py3-none-any.whl",  # excludes pre-releases
+        requests.get("https://pypi.org/simple/pyodide-build/").text,
+    )
+    pyodide_build_version = max(
+        # Don't just pick the most recent version; find the highest stable version.
+        set(all_pyodide_build_versions),
+        key=version_tuple,
+    )
+
+    cross_build_environments_url = "https://raw.githubusercontent.com/pyodide/pyodide/refs/heads/main/pyodide-cross-build-environments.json"
+    cross_build_environments_data = requests.get(cross_build_environments_url).json()
+
+    # Find the latest stable release for the Pyodide runtime/xbuildenv that is compatible
+    # with the pyodide-build version we found
+    stable_releases = [
+        release
+        for release in cross_build_environments_data["releases"].values()
+        if re.fullmatch(vers_re, release["version"])
+    ]
+
+    compatible_releases = []
+    for release in stable_releases:  # sufficiently large values
+        min_build_version = release.get("min_pyodide_build_version", "0.0.0")
+        max_build_version = release.get("max_pyodide_build_version", "999.999.999")
+
+        # Perform version comparisons to avoid getting an incompatible pyodide-build version
+        # with the Pyodide runtime
+        if (
+            version_tuple(min_build_version)
+            <= version_tuple(pyodide_build_version)
+            <= version_tuple(max_build_version)
+        ):
+            compatible_releases.append(release)
+
+    if not compatible_releases:
+        raise RuntimeError(
+            f"No compatible Pyodide release found for pyodide-build {pyodide_build_version}"
+        )
+
+    pyodide_release = max(
+        compatible_releases,
+        key=lambda release: version_tuple(release["version"]),
+    )
+
+    pyodide_version = pyodide_release["version"]
+    python_version = pyodide_release["python_version"]
+
+    ci_file = tools.ROOT / ".github/workflows/main.yml"
+    config = ci_file.read_text(encoding="utf-8")
+    for name, var in [
+        ("PYODIDE", pyodide_version),
+        ("PYODIDE_BUILD", pyodide_build_version),
+        ("PYTHON", python_version),
+    ]:
+        config = re.sub(f"{name}_VERSION: {vers_re}", f"{name}_VERSION: {var}", config)
+    ci_file.write_text(config, encoding="utf-8")
+
+
 def update_vendored_files():
     vendor = pathlib.Path(hp.PYTHON_SRC) / "hypothesis" / "vendor"
 
@@ -311,12 +482,16 @@ def update_vendored_files():
     if fname.read_bytes().splitlines()[1:] != new.splitlines()[1:]:
         fname.write_bytes(new)
 
-    # Always require the latest version of the tzdata package
-    tz_url = "https://pypi.org/pypi/tzdata/json"
-    tzdata_version = requests.get(tz_url).json()["info"]["version"]
-    setup = pathlib.Path(hp.BASE_DIR, "setup.py")
-    new = re.sub(r"tzdata>=(.+?) ", f"tzdata>={tzdata_version} ", setup.read_text())
-    setup.write_text(new)
+    # Always require the most recent version of tzdata - we don't need to worry about
+    # pre-releases because tzdata is a 'latest data' package  (unlike pyodide-build).
+    # Our crosshair extra is research-grade, so we require latest versions there too.
+    pyproject_toml = pathlib.Path(hp.BASE_DIR, "pyproject.toml")
+    new = pyproject_toml.read_text(encoding="utf-8")
+    for pkgname in ("tzdata", "crosshair-tool", "hypothesis-crosshair"):
+        pkg_url = f"https://pypi.org/pypi/{pkgname}/json"
+        pkg_version = requests.get(pkg_url).json()["info"]["version"]
+        new = re.sub(rf"{pkgname}>=([a-z0-9.]+)", f"{pkgname}>={pkg_version}", new)
+    pyproject_toml.write_text(new, encoding="utf-8")
 
 
 def has_diff(file_or_directory):
@@ -331,9 +506,11 @@ def upgrade_requirements():
     subprocess.call(["./build.sh", "format"], cwd=tools.ROOT)  # exits 1 if changed
     if has_diff(hp.PYTHON_SRC) and not os.path.isfile(hp.RELEASE_FILE):
         msg = hp.get_autoupdate_message(domainlist_changed=has_diff(hp.DOMAINS_LIST))
-        with open(hp.RELEASE_FILE, mode="w") as f:
-            f.write(f"RELEASE_TYPE: patch\n\n" + msg)
+        with open(hp.RELEASE_FILE, mode="w", encoding="utf-8") as f:
+            f.write(f"RELEASE_TYPE: patch\n\n{msg}")
     update_python_versions()
+    update_pyodide_versions()
+    update_django_versions()
     subprocess.call(["git", "add", "."], cwd=tools.ROOT)
 
 
@@ -350,11 +527,21 @@ def documentation():
         hp.build_docs()
     finally:
         subprocess.check_call(
-            ["git", "checkout", "docs/changes.rst", "src/hypothesis/version.py"],
+            ["git", "checkout", "docs/changelog.rst", "src/hypothesis/version.py"],
             cwd=hp.HYPOTHESIS_PYTHON,
         )
 
 
+@task()
+def website():
+    subprocess.call(["pelican"], cwd=tools.ROOT / "website")
+
+
+@task()
+def live_website():
+    subprocess.call(["pelican", "--autoreload", "--listen"], cwd=tools.ROOT / "website")
+
+
 def run_tox(task, version, *args):
     python = install.python_executable(version)
 
@@ -377,17 +564,19 @@ def run_tox(task, version, *args):
 
 # update_python_versions(), above, keeps the contents of this dict up to date.
 # When a version is added or removed, manually update the env lists in tox.ini and
-# workflows/main.yml, and the `Programming Language ::` specifiers in setup.py
+# workflows/main.yml, and the `Programming Language ::` specifiers in pyproject.toml
 PYTHONS = {
-    "3.7": "3.7.16",
-    "3.8": "3.8.16",
-    "3.9": "3.9.16",
-    "3.10": "3.10.10",
-    "3.11": "3.11.2",
-    "3.12": "3.12-dev",
-    "pypy3.7": "pypy3.7-7.3.9",
-    "pypy3.8": "pypy3.8-7.3.11",
-    "pypy3.9": "pypy3.9-7.3.11",
+    "3.9": "3.9.22",
+    "3.10": "3.10.17",
+    "3.11": "3.11.12",
+    "3.12": "3.12.10",
+    "3.13": "3.13.3",
+    "3.13t": "3.13t-dev",
+    "3.14": "3.14.0a7",
+    "3.14t": "3.14t-dev",
+    "pypy3.9": "pypy3.9-7.3.16",
+    "pypy3.10": "pypy3.10-7.3.19",
+    "pypy3.11": "pypy3.11-7.3.19",
 }
 ci_version = "3.10"  # Keep this in sync with GH Actions main.yml and .readthedocs.yml
 
@@ -396,6 +585,7 @@ python_tests = task(
         hp.PYTHON_SRC,
         hp.PYTHON_TESTS,
         os.path.join(tools.ROOT, "pytest.ini"),
+        os.path.join(tools.ROOT, "tooling"),
         os.path.join(hp.HYPOTHESIS_PYTHON, "scripts"),
     )
 )
@@ -432,9 +622,9 @@ def tox(*args):
     run_tox(*args)
 
 
-def standard_tox_task(name, *args, py=ci_version):
+def standard_tox_task(name, py=ci_version):
     TASKS["check-" + name] = python_tests(
-        lambda: run_tox(name, PYTHONS.get(py, py), *args)
+        lambda *args: run_tox(name, PYTHONS.get(py, py), *args)
     )
 
 
@@ -443,12 +633,22 @@ standard_tox_task("py39-pytest46", py="3.9")
 standard_tox_task("py39-pytest54", py="3.9")
 standard_tox_task("pytest62")
 
-for n in [32, 40, 41]:
-    standard_tox_task(f"django{n}")
+for n in DJANGO_VERSIONS:
+    standard_tox_task(f"django{n.replace('.', '')}")
+# we also test no-contrib on the latest django version
+standard_tox_task("django-nocontrib")
 
-standard_tox_task("py39-pandas10", py="3.9")
-for n in [11, 12, 13, 14, 15]:
+for n in [13, 14, 15, 20, 21, 22]:
     standard_tox_task(f"pandas{n}")
+standard_tox_task("py39-pandas11", py="3.9")
+standard_tox_task("py39-pandas12", py="3.9")
+
+for kind in ("cover", "nocover", "niche", "custom"):
+    standard_tox_task(f"crosshair-{kind}")
+
+standard_tox_task("py39-oldestnumpy", py="3.9")
+standard_tox_task("py39-oldparser", py="3.9")
+standard_tox_task("numpy-nightly", py="3.12")
 
 standard_tox_task("coverage")
 standard_tox_task("conjecture-coverage")
@@ -472,7 +672,7 @@ def check_whole_repo_tests(*args):
     )
 
     if not args:
-        args = [tools.REPO_TESTS]
+        args = ["-n", "auto", tools.REPO_TESTS]
     subprocess.check_call([sys.executable, "-m", "pytest", *args])
 
 
@@ -521,7 +721,7 @@ def audit_rust_in_ruby():
 
 @task()
 def python(*args):
-    os.execv(sys.executable, (sys.executable,) + args)
+    os.execv(sys.executable, (sys.executable, *args))
 
 
 @task()
